@@ -24,7 +24,7 @@
   * @author Samuel Mimram
   *)
 
-(* $Id: mad.mli 2919 2007-03-02 14:25:16Z metamorph68 $ *)
+(* $Id: mad.mli 4672 2007-10-17 16:48:18Z metamorph68 $ *)
 
 (** {1 Exceptions} *)
 
@@ -86,15 +86,26 @@ val close : mad_file -> unit
 (**
   * Get the current position (in bytes) of the decoder in the mp3 file which
   * should have been opened with [openfile].
-  **)
+  *)
 val get_current_position : mad_file -> int
 
 (** Decode an mp3 frame. *)
 val decode_frame : mad_file -> string
 
+(** Decode an mp3 frame. *)
+val decode_frame_float : mad_file -> float array array
+
+(*
+ * Get the samplerate, number of channels and samples per channel currently in
+ * the synth. This should be called after [decode_frame] or
+ * [decode_frame_float].
+ *)
+val get_output_format : mad_file -> int * int * int
+
 (** {1 Some info about MP3 files} *)
 
-(** Get the samplefreq of a file. *)
+(** Get the samplefreq of a file.
+  * This is a dirty hack, you should prefer get_output_format. *)
 val samplefreq : string -> int
 
 (** Compute the duration of a file, in seconds.
