@@ -24,7 +24,7 @@
  * @author Samuel Mimram, David Baelde
  */
 
-/* $Id: mad_stubs.c 2919 2007-03-02 14:25:16Z metamorph68 $ */
+/* $Id: mad_stubs.c 4672 2007-10-17 16:48:18Z metamorph68 $ */
 
 #include <caml/alloc.h>
 #include <caml/callback.h>
@@ -64,7 +64,7 @@ typedef struct madfile__t madfile_t;
  *
  * Shamelessly taken from the MPlayer source code.
  */
-static unsigned short ushort_of_madfixed(mad_fixed_t sample)
+static short short_of_madfixed(mad_fixed_t sample)
 {
   /* A fixed point number is formed of the following bit pattern:
    *
@@ -96,7 +96,7 @@ static unsigned short ushort_of_madfixed(mad_fixed_t sample)
   /* quantize */
   return sample >> (MAD_F_FRACBITS + 1 - 16);
 
-  return ((unsigned short)sample);
+  return (short)sample;
 }
 
 #define Madfile_val(v) (*((madfile_t**)Data_custom_val(v)))
@@ -199,14 +199,10 @@ CAMLprim value ocaml_mad_get_current_position(value madf)
 /*
  * Shamelessly inspired of the Madlld source code.
  */
-CAMLprim value ocaml_mad_decode_frame(value madf)
+static void mf_fill_buffer(madfile_t *mf)
 {
-  CAMLparam1(madf);
-  CAMLlocal2(data, ret);
-  madfile_t *mf = Madfile_val(madf);
-  char *output_buf = NULL;
-  int output_pos = 0;
-  int i, dec;
+  CAMLparam0();
+  CAMLlocal1(data);
 
   /* The input bucket must be filled if it becomes empty
   */
@@ -315,7 +311,14 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
     mf->stream.error = 0;
   }
 
-  /***** ENTER BLOCKING SECTION *****/
+  CAMLreturn0;
+}
+
+/* Returns 1 if a recoverable error occured, 0 else. */
+static int mf_decode(madfile_t *mf)
+{
+  int dec;
+
   caml_enter_blocking_section();
 
   /* Decode the next mpeg frame. The streams is read from the
@@ -351,7 +354,7 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
     if (MAD_RECOVERABLE(mf->stream.error)
         || mf->stream.error == MAD_ERROR_BUFLEN) {
       caml_leave_blocking_section();
-      CAMLreturn(ocaml_mad_decode_frame(madf));
+      return 1;
     } else {
       caml_leave_blocking_section();
       caml_raise_with_arg(*caml_named_value("mad_exn_mad_error"), caml_copy_string(mad_stream_errorstr(&mf->stream)));
@@ -374,8 +377,22 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
    */
   mad_synth_frame(&mf->synth, &mf->frame);
 
-  /***** LEAVE BLOCKING SECTION *****/
   caml_leave_blocking_section();
+  return 0;
+}
+
+CAMLprim value ocaml_mad_decode_frame(value madf)
+{
+  CAMLparam1(madf);
+  CAMLlocal1(ret);
+  madfile_t *mf = Madfile_val(madf);
+  char *output_buf = NULL;
+  int output_pos = 0;
+  int i;
+
+  mf_fill_buffer(mf);
+  if(mf_decode(mf))
+    ocaml_mad_decode_frame(madf);
 
   /* Synthesized samples must be converted from mad's fixed
    * point number to the consumer format. Here we use unsigned
@@ -391,7 +408,7 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
     unsigned short sample;
 
     /* Left channel */
-    sample = ushort_of_madfixed(mf->synth.pcm.samples[0][i]);
+    sample = short_of_madfixed(mf->synth.pcm.samples[0][i]);
     *(output_buf + output_pos + 0) = sample & 0xff;
     *(output_buf + output_pos + 1) = sample >> 8;
 
@@ -399,7 +416,7 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
      * the right output channel is the same as the left one.
      */
     if (MAD_NCHANNELS(&mf->frame.header) == 2)
-      sample = ushort_of_madfixed(mf->synth.pcm.samples[1][i]);
+      sample = short_of_madfixed(mf->synth.pcm.samples[1][i]);
     *(output_buf + output_pos + 2) = sample & 0xff;
     *(output_buf + output_pos + 3) = sample >> 8;
 
@@ -408,3 +425,42 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
 
   CAMLreturn(ret);
 }
+
+CAMLprim value ocaml_mad_decode_frame_float(value madf)
+{
+  CAMLparam1(madf);
+  CAMLlocal1(ret);
+  madfile_t *mf = Madfile_val(madf);
+  int chans;
+  int i, c;
+
+  mf_fill_buffer(mf);
+  if(mf_decode(mf))
+    ocaml_mad_decode_frame_float(madf);
+
+  chans = MAD_NCHANNELS(&mf->frame.header);
+  ret = caml_alloc_tuple(chans);
+
+  for (c = 0; c < chans; c++)
+    Store_field(ret, c, caml_alloc(mf->synth.pcm.length * Double_wosize, Double_array_tag));
+
+  for (c = 0; c < chans; c++)
+    for (i = 0; i < mf->synth.pcm.length; i++)
+      Store_double_field(Field(ret, c), i, mad_f_todouble(mf->synth.pcm.samples[c][i]));
+
+  CAMLreturn(ret);
+}
+
+CAMLprim value ocaml_mad_get_synth_pcm_format(value madf)
+{
+  CAMLparam1(madf);
+  CAMLlocal1(ans);
+  madfile_t *mf = Madfile_val(madf);
+
+  ans = caml_alloc_tuple(3);
+  Store_field(ans, 0, Val_int(mf->synth.pcm.samplerate));
+  Store_field(ans, 1, Val_int(mf->synth.pcm.channels));
+  Store_field(ans, 2, Val_int(mf->synth.pcm.length));
+
+  CAMLreturn(ans);
+}
